package com.company.kotlinlateinit.app

import com.company.kotlinlateinit.entity.User
import io.jmix.core.DataManager
import io.jmix.core.FetchPlan
import io.jmix.core.security.SystemAuthenticator
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Component
import java.util.concurrent.TimeUnit

@Component("REG_ScheduledTasks")
open class ScheduledTasks(private val systemAuthenticator: SystemAuthenticator) {

    @Autowired
    private lateinit var dataManager: DataManager

    companion object {
        private val log = LoggerFactory.getLogger(ScheduledTasks::class.java)
    }

    @Scheduled(fixedDelay = 10, timeUnit = TimeUnit.SECONDS)
    fun runCheckScheduledTasks() {
        log.info("Scheduled task started")

        systemAuthenticator.runWithSystem {
            val users = dataManager.load(User::class.java)
                .query("select t from User t")
                .fetchPlan(FetchPlan.BASE)
                .list()
            log.info("Users: {}", users.size)
        }
        log.info("runCheckScheduledTasks: done.")
    }
}